#include "classfile.h"
#include "lowlevel.h"
#include <stdio.h>
#include <fcntl.h>
#include <string.h>
#include <unistd.h>

const uint32_t magic = 0xcafebabe;

TClassFile classFile;

/* Read constant pool entries */
void readCpInfo(int fd, Tcp_info *info, int *i)
{
    info[*i].tag = read_u1(fd);
    switch(info[*i].tag)
    {
        case Class:
            info[*i].info.class_info.name_index = read_u2(fd);
            break;
        case Fieldref:
            info[*i].info.fieldref_info.class_index = read_u2(fd);
            info[*i].info.fieldref_info.name_and_type_index = read_u2(fd);
            break;
        case Methodref:
            info[*i].info.fieldref_info.class_index = read_u2(fd);
            info[*i].info.fieldref_info.name_and_type_index = read_u2(fd);
            break;
        case InterfaceMethodref:
            info[*i].info.fieldref_info.class_index = read_u2(fd);
            info[*i].info.fieldref_info.name_and_type_index = read_u2(fd);
            break;
        case String:
            info[*i].info.string_info.string_index = read_u2(fd);
            break;
        case Integer:
            info[*i].info.integer_info.bytes = read_u4(fd);
            break;
        case Float:
            info[*i].info.float_info.bytes = read_u4(fd);
            break;
        case Long:
            info[*i].info.long_info./*high_*/bytes = read_u4(fd);
            (*i)++;
            info[*i].info.long_info./*low_*/bytes = read_u4(fd);
            break;
        case Double:
            info[*i].info.double_info./*high_*/bytes = read_u4(fd);
            (*i)++;
            info[*i].info.double_info./*low_*/bytes = read_u4(fd);
            break;
        case NameAndType:
            info[*i].info.nameandtype_info.name_index = read_u2(fd);
            info[*i].info.nameandtype_info.descriptor_index = read_u2(fd);
            break;
        case Utf8:
            info[*i].info.utf8_info.length = read_u2(fd);
            info[*i].info.utf8_info.bytes = newTC(uint8_t, info[*i].info.utf8_info.length);
            read(fd, info[*i].info.utf8_info.bytes, info[*i].info.utf8_info.length);
            break;
        default:
            fprintf(stderr, "Unknown constant_pool_tag: %d\n", info[*i].tag);
            exit(1);
            break;
    }
}

/* Read attribute */
void readAttributeInfo(int fd, Tattribute_info *info)
{
    void *tmp;
    TCONSTANT_Utf8_info type;
    int i, l; char *c;

    info->attribute_name_index = i = read_u2(fd);
    info->attribute_length = read_u4(fd);

    type = classFile.constant_pool[i].info.utf8_info;
    c = (char *)type.bytes;
    if((strncmp(c, ISourceFile_attribute, type.length) == 0))
    {
        info->info.sourcefile.sourcefile_index = read_u2(fd);
    }
    else if((strncmp(c, IConstantValue_attribute, type.length) == 0))
    {
        info->info.constantvalue.constantvalue_index = read_u2(fd);
    }
    else if((strncmp(c, ICode_attribute, type.length) == 0))
    {
        info->info.code.max_stack = read_u2(fd);
        info->info.code.max_locals = read_u2(fd);

        info->info.code.code_length = l = read_u4(fd);
        info->info.code.code =  newTC(uint8_t, l);
        read(fd, info->info.code.code, l);

        info->info.code.exception_table_length = l = read_u2(fd);
        info->info.code.exception_table = newTC(TJexceptions, l);
        for(i=0; i<l; i++)
        {
            info->info.code.exception_table[i].start_pc = read_u2(fd);
            info->info.code.exception_table[i].end_pc = read_u2(fd);
            info->info.code.exception_table[i].handler_pc = read_u2(fd);
            info->info.code.exception_table[i].catch_type = read_u2(fd);
        }

        info->info.code.attributes_count = l = read_u2(fd);
        info->info.code.attributes = newTC(Tattribute_info, l);
        for(i=0; i<l; i++)
            readAttributeInfo(fd, info->info.code.attributes+i);
    }
    else if((strncmp(c, IExceptions_attribute, type.length) == 0))
    {
        info->info.exceptions.number_of_exceptions = l = read_u2(fd);
        info->info.exceptions.exception_index_table = newTC(uint16_t, l);
        for(i=0; i<l; i++)
            info->info.exceptions.exception_index_table[i] = read_u2(fd);
    }
    else if((strncmp(c, ILineNumberTable_attribute, type.length) == 0))
    {
        info->info.linenumbertable.line_number_table_length = l = read_u2(fd);
        info->info.linenumbertable.line_number_table = newTC(TJlinenumbertable, l);
        for(i=0; i<l; i++)
        {
            info->info.linenumbertable.line_number_table[i].start_pc = read_u2(fd);
            info->info.linenumbertable.line_number_table[i].line_number = read_u2(fd);
        }
    }
    else if((strncmp(c, ILocalVariableTable_attribute, type.length) == 0))
    {
        info->info.localvariabletable.local_variable_table_length = l = read_u2(fd);
        info->info.localvariabletable.local_variable_table = newTC(TJlocalvariabletable, l);
        for(i=0; i<l; i++)
        {
            info->info.localvariabletable.local_variable_table[i].start_pc = read_u2(fd);
            info->info.localvariabletable.local_variable_table[i].length = read_u2(fd);
            info->info.localvariabletable.local_variable_table[i].name_index = read_u2(fd);
            info->info.localvariabletable.local_variable_table[i].descriptor_index = read_u2(fd);
            info->info.localvariabletable.local_variable_table[i].index = read_u2(fd);
        }
    }
    else if((strncmp(c, IInnerClasses_attribute, type.length) == 0))
    {
        info->info.innerclasses.number_of_classes = l = read_u2(fd);
        info->info.innerclasses.classes = newTC(Tinner_class_info, l);
        for(i=0; i<l; i++)
        {
            info->info.innerclasses.classes[i].inner_class_info_index = read_u2(fd);
            info->info.innerclasses.classes[i].outer_class_info_index = read_u2(fd);
            info->info.innerclasses.classes[i].inner_name_index = read_u2(fd);
            info->info.innerclasses.classes[i].inner_class_access_flags = read_u2(fd);
        }
    }
    else if((strncmp(c, ISynthetic_attribute, type.length) == 0))
    {
    }
    else if((strncmp(c, IDeprecated_attribute, type.length) == 0))
    {
    }
    else
    {
        fprintf(stderr, "Unknown attribute: ");
        for(i=type.length; i--; c++) putc(*c, stderr);
        putc('\n', stderr);
        tmp = Malloc(info->attribute_length);
        read(fd, tmp, info->attribute_length);
        free(tmp);
    }
}

/* Read field */
void readFieldsInfo(int fd, Tfield_info *info)
{
    int i, l;

    info->access_flags = read_u2(fd);
    info->name_index = read_u2(fd);
    info->descriptor_index = read_u2(fd);
    info->attributes_count = l = read_u2(fd);
    info->attributes = newTC(Tattribute_info, l);
    for(i=0; i<l; i++)
        readAttributeInfo(fd, info->attributes+i);
}

/* Read method */
void readMethodInfo(int fd, Tmethod_info *info)
{
    int i, l;

    info->access_flags = read_u2(fd);
    info->name_index = read_u2(fd);
    info->descriptor_index = read_u2(fd);
    info->attributes_count = l = read_u2(fd);
    info->attributes = newTC(Tattribute_info, l);
    for(i=0; i<l; i++)
        readAttributeInfo(fd, info->attributes+i);
}

/* Read whole class file */
void readClassFile(int fd)
{
    int i, l;

    classFile.magic = read_u4(fd);
    classFile.minor_version = read_u2(fd);
    classFile.major_version = read_u2(fd);
    classFile.constant_pool_count = l = read_u2(fd);
    classFile.constant_pool = newTC(Tcp_info, l);
    classFile.constant_pool[0].tag = 0; /* Speciel */
    for(i=1; i<l; i++)
        readCpInfo(fd, classFile.constant_pool, &i);

    classFile.access_flags = read_u2(fd);
    classFile.this_class = read_u2(fd);
    classFile.super_class = read_u2(fd);

    classFile.interfaces_count = l = read_u2(fd);
    classFile.interfaces = newTC(uint16_t, l);
    for(i=0; i<l; i++)
        classFile.interfaces[i] = read_u2(fd);

    classFile.fields_count = l = read_u2(fd);
    classFile.fields = newTC(Tfield_info, l);
    for(i=0; i<l; i++)
        readFieldsInfo(fd, classFile.fields+i);

    classFile.methods_count = l = read_u2(fd);
    classFile.methods = newTC(Tmethod_info, l);
    for(i=0; i<l; i++)
        readMethodInfo(fd, classFile.methods+i);

    classFile.attributes_count = l = read_u2(fd);
    classFile.attributes = newTC(Tattribute_info, l);
    for(i=0; i<l; i++)
        readAttributeInfo(fd, classFile.attributes+i);
}

#if 0
int main(void)
{
    int fd;
    fd = open("/home/pmhahn/prog/Descriptor.class", O_RDONLY);
    readClassFile(fd);
    printf("%ld\n", lseek(fd, 0, SEEK_CUR));
    printf("%ld\n", lseek(fd, 0, SEEK_END));
    close(fd);
    return 0;
}
#endif
/* vim: autoindent expandtab tabstop=4 shiftwidth=4
*/
