#include <stdio.h>
#include <fcntl.h>
#include <stdlib.h>
#include <stdint.h>
#include <string.h>
#include <unistd.h>

#include "classfile.h"
#include "lowlevel.h"

/* Main program */
int main(int argc, char **argv)
{
    int fd;
    uint16_t len;
    char *class;
    char *c;
    char *dir;
    int i, j;
    char *env;
    Tcp_info cpinfo;

    Malloc_init(8);

    /* Dateiname */
    if(argc < 2)
        ERROR("Kein dateiname angegeben\n", 1);
    /* Datei lesem */
    if((fd = open(argv[1], O_RDONLY)) < 0)
        ERROR("Datei kann nicht geoeffnet werden\n", 2);
    readClassFile(fd);
    close(fd);

    /* Magic cafebabe teste */
    if(classFile.magic != magic)
        ERROR("Datei ist keine Java-Klasse\n", 3);

    /* Klasse suchen */
    i = classFile.this_class;
    if((i < 1) || (i >= classFile.constant_pool_count))
        ERROR("Klassenreferenz ist ausserhalb des gueltigen Bereichs\n", 4);
    cpinfo = classFile.constant_pool[i];
    if(cpinfo.tag != Class)
        ERROR("Klassenreferenz beschreibt keine Klasse\n", 5);
    /* Klassenname suchen */
    j = cpinfo.info.class_info.name_index;
    if((j < 1) || (j >= classFile.constant_pool_count))
        ERROR("Klassenname ist ausserhalb des gueltigen Bereichs\n", 6);
    cpinfo = classFile.constant_pool[j];
    if(cpinfo.tag != Utf8)
        ERROR("Klassenname ist keine Zeichenkette\n", 7);

    /* Klassenname lesen */
    len = cpinfo.info.utf8_info.length;
    class = (char *)malloc(len+1);
    memmove(class, cpinfo.info.utf8_info.bytes, len);
    class[len] = 0;

    /* Basisverzeichnis extrahieren */
    dir = strdup(argv[1]);
    if((c = strstr(dir, class)) == NULL)
        ERROR("Datei ist nicht im richtigen Verzeichnis\n", 8);
    /* Dateiname abschneiden */
    *c = 0;
    /* Dateiname in Klassenname umwandeln */
    for(c = class; c[0]; c++)
        if(c[0] == '/')
            c[0] = '.';

    /* Umgebung erweitern */
    if((env = getenv(CLASSPATH)))
    {
        i = strlen(env);
        j = strlen(dir);
        c = (char *)Malloc(i + 1 + j + 1);
        strncpy(c, env, i);
        c[i] = ':';
        strncpy(c+i+1, dir, j);
        c[i+1+j] = 0;
        env = c;
    }
    else
    {
        env = dir;
    }
    if(setenv(CLASSPATH, env, 1))
        ERROR("CLASSPATH kann nicht angepasst werden\n", 9);

    Malloc_done();
    
    /* Argument anpassen */
    argv[0] = "java";
    argv[1] = class;
    /* Java starten */
    return execvp("java", argv);
}
/* vim: autoindent expandtab tabstop=4 shiftwidth=4
*/
