#include "lowlevel.h"

/* Reads 8bit unsigned word */
uint8_t read_u1(int fd)
{
    uint8_t data;

    read(fd, &data, 1);
    return data;
}

/* Reads 16bit unsigned word */
uint16_t read_u2(int fd)
{
    uint16_t data;

    read(fd, &data, 2);
#if __BYTE_ORDER == __LITTLE_ENDIAN
    data = bswap_16(data);
#elif __BYTE_ORDER == __BIG_ENDIAN
#else
#    error "Unbekanntes Format."
#endif
    return data;
}

/* Reads 32bit unsigned word */
uint32_t read_u4(int fd)
{
    uint32_t data;

    read(fd, &data, 4);
#if __BYTE_ORDER == __LITTLE_ENDIAN
    data = bswap_32(data);
#elif __BYTE_ORDER == __BIG_ENDIAN
#else
#    error "Unbekanntes Format."
#endif
    return data;
}

#if defined CLEAN_MEMORY
int memory_table_size = 0;
int memory_table_used = 0;
void **memory_table = NULL;

/* Allocate memory and remeber pointer */
void *Malloc(size_t size)
{
    void *ptr = malloc(size);
    if(memory_table_used >= memory_table_size)
    {
        memory_table_size <<= 1;
        memory_table = realloc(memory_table, memory_table_size * sizeof(void *));
    }
    memory_table[memory_table_used++] = ptr;
    return ptr;
}

/* Initialize memory allocator */
void Malloc_init(int size)
{
    memory_table = malloc(size * sizeof(void *));
    memory_table_size = size;
    memory_table_used = 0;
}

/* Free allocated memory */
void Malloc_done()
{
    int i;
    for(i = 0; i < memory_table_used; i++)
        free(memory_table[i]);
    free(memory_table);
}
#endif

/* vim: autoindent expandtab tabstop=4 shiftwidth=4
*/
